import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import os

import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

directory = './data/figure3'

os.makedirs('./output/', exist_ok=True)

# figure 3a
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns

fig, axes = plt.subplots(4, 4, sharex=True, sharey=True, dpi=200, figsize=(5,5))
cmap = sns.color_palette('colorblind')

cols = [f't = {i} ms' for i in [1, 1.5, 2, 2.5]]
rows = [f'-{i} mA' for i in [0.012, 0.19, 0.2, 0.27]]

t_indices = [200, 300, 400, 500]

y = np.load(f'{directory}/a_y_vals.npy')

for i in range(4):
    states_mrggpu = np.load(f'{directory}/a_stim_{i}.npy')
    states_nrn = np.load(f'{directory}/a_stim_{i}_nrn.npy')
    for j in range(4):
        t_indx = t_indices[j]
        axes[i, j].plot(y/1000, states_nrn[:, t_indx], c=cmap[0], label='NEURON')
        axes[i, j].plot(y/1000, states_mrggpu[t_indx, 0, -1, :], c=cmap[1], linestyle='--', label='S-MF')

handles, labels = plt.gca().get_legend_handles_labels()
#fig.legend(handles, labels, loc='right')

fig.add_subplot(111, frameon=False)
# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', which='both', top=False, bottom=False, left=False, right=False)
plt.xlabel("position along axon (mm)")
plt.ylabel("$V_m$ (mV)", labelpad = 12)
#axes[0, 3].legend(loc='center left', bbox_to_anchor=(1, 0.5))

for ax, col in zip(axes[0], cols):
    ax.set_title(col, size='medium')

for ax, row in zip(axes[:,0], rows):
    ax.set_ylabel(row, rotation=45, size='medium', labelpad=35)
    
from matplotlib.lines import Line2D

custom_lines = [Line2D([0], [0], color=cmap[0], lw=2),
                Line2D([0], [0], color=cmap[1], lw=2, linestyle='--')]
plt.gca().legend(custom_lines, ['NEURON', 'S-MF'], loc='upper center', bbox_to_anchor=(0.5, -0.12), ncols=2)
fig.savefig('./output/fig3a.svg', bbox_inches='tight')


# figure 3b

data_df = pd.read_csv(f'{directory}/figure3b.csv')

all_diams = [5.7, 8.7, 14.0]
frequencies = [1, 2, 5, 10]

cols = ['5.7 $\mu m$', '8.7 $\mu m$', '14.0 $\mu m$']
rows = ['1 kHz', '2 kHz', '5 kHz', '10 kHz']

fig, axes = plt.subplots(4, 3, dpi=200, figsize=(5, 5), sharex='col', sharey=True)

for i, d_selec in enumerate(all_diams):
    for j, f_selec in enumerate(frequencies):
        data_selection = data_df[data_df['diameter']==d_selec]
        data_selection = data_selection[data_selection['frequency']==f_selec]
        sns.lineplot(data=data_selection, 
                     x='amplitude', 
                     y='# APs', 
                     hue='method', 
                     hue_order=['NEURON', 'Surrogate'],
                     legend=False, 
                     palette='colorblind', 
                     ax=axes[j, i], 
                     errorbar='sd')
        axes[j, i].set_ylabel('')
        axes[j, i].set_xlabel('')

fig.add_subplot(111, frameon=False)
# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', 
                which='both', 
                top=False, 
                bottom=False, 
                left=False, 
                right=False)
plt.xlabel("stimulus amplitude (mA)")
plt.ylabel("# APs", labelpad = 5)

for ax, col in zip(axes[0], cols):
    ax.set_title(col, size='medium')

for ax, row in zip(axes[:,0], rows):
    ax.set_ylabel(row, rotation=45, size='medium', labelpad=35)
    
from matplotlib.lines import Line2D
cmap = sns.color_palette('colorblind')
custom_lines = [Line2D([0], [0], color=cmap[0], lw=2),
                Line2D([0], [0], color=cmap[1], lw=2)]
plt.gca().legend(custom_lines, 
                 ['NEURON', 'S-MF'],
                 loc='upper center', 
                 bbox_to_anchor=(0.5, -0.12), 
                 ncols=2)

fig.savefig('./output/fig3b.svg', bbox_inches='tight')


# figure 3c
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns

fig, axes = plt.subplots(4, 5, sharex=True, sharey=True, dpi=200, figsize=(5,5))
cmap = sns.color_palette('colorblind')

cols = [f't = {i} ms' for i in [0.125, 0.5, 1.0, 1.5, 2.0]]
rows = [f'{i} $\mu m$' for i in [5.7, 8.7, 10, 14]]

ds = [5.7, 8.7, 10, 14]
indices = [25, 100, 200, 300, 400]
t_indices = indices

surr_data = np.load(f'{directory}/c_surr_collision.npy')

for i in range(4):
    states_mrggpu = surr_data[:, i, :, :]
    states_nrn = np.load(f'{directory}/c_nrn_{i}.npy')
    for j in range(5):
        t_indx = t_indices[j]
        axes[i, j].plot(states_nrn[:, t_indx], c=cmap[0], label='NEURON')
        axes[i, j].plot(states_mrggpu[t_indx, -1, :], c=cmap[1], linestyle='--', label='Surrogate')

handles, labels = plt.gca().get_legend_handles_labels()
#fig.legend(handles, labels, loc='right')

fig.add_subplot(111, frameon=False)
# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', which='both', top=False, bottom=False, left=False, right=False)
plt.xlabel("position along axon (node #)")
plt.ylabel("$V_m$ (mV)", labelpad = 12)
#axes[0, 3].legend(loc='center left', bbox_to_anchor=(1, 0.5))

for ax, col in zip(axes[0], cols):
    ax.set_title(col, size='small')

for ax, row in zip(axes[:,0], rows):
    ax.set_ylabel(row, rotation=45, size='medium', labelpad=35)
    
from matplotlib.lines import Line2D

custom_lines = [Line2D([0], [0], color=cmap[0], lw=2),
                Line2D([0], [0], color=cmap[1], lw=2, linestyle='--')]
plt.gca().legend(custom_lines, ['NEURON', 'S-MF'], loc='upper center', bbox_to_anchor=(0.5, -0.12), ncols=2)
fig.savefig('./output/fig3c.svg', bbox_inches='tight')


# figure 3d

import seaborn as sns
import matplotlib.pyplot as plt

data_df = pd.read_csv(f'{directory}/figure3d.csv')

pw = 0.1
spk_is = list(range(5))
spk_freqs = [10, 50, 100]
ex_freqs = [30, 50, 100]
diameters = [5.7, 8.7, 14.0]

cols = ['5.7 $\mu m$', '8.7 $\mu m$', '14.0 $\mu m$']
rows = ['10 Hz', '50 Hz', '100 Hz']

fig, axes = plt.subplots(3, 3, dpi=200, figsize=(5, 5), sharex='col', sharey=True)

for i, d_selec in enumerate(diameters):
    for j, f_selec in enumerate(spk_freqs):
        data_selection = data_df[data_df['diameter']==d_selec]
        data_selection = data_selection[data_selection['IFR']==f_selec]
        sns.lineplot(data=data_selection, 
                     x='amp', 
                     y='sync', 
                     hue='method', 
                     style='stimulation frequency', 
                     style_order=[30, 50, 100], 
                     hue_order=['NEURON', 'Surrogate'],
                     legend=False, 
                     palette='colorblind',
                     ax=axes[j, i], 
                     errorbar='ci')
        axes[j, i].set_ylabel('')
        axes[j, i].set_xlabel('')

fig.add_subplot(111, frameon=False)

# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', which='both', top=False, bottom=False, left=False, right=False)
plt.xlabel("stimulus amplitude (mA)")
plt.ylabel("SPIKE synchronization", labelpad = 10)

for ax, col in zip(axes[0], cols):
    ax.set_title(col, size='medium')

for ax, row in zip(axes[:,0], rows):
    ax.set_ylabel(row, rotation=45, size='medium', labelpad=35)
    
from matplotlib.lines import Line2D
cmap = sns.color_palette('colorblind')
custom_lines = [Line2D([], [], color="white", marker='o', markerfacecolor=cmap[0]),
                Line2D([], [], color="white", marker='o', markerfacecolor=cmap[1]),
                Line2D([0], [0], color='black', lw=2, linestyle='-'),
                Line2D([0], [0], color='black', lw=2, linestyle='--'),
                Line2D([0], [0], color='black', lw=2, linestyle=':')]

names = ['NEURON', 'S-MF', '30 Hz', '50 Hz',  '100 Hz']
plt.gca().legend(custom_lines, 
                 names, 
                 loc='upper center',
                 bbox_to_anchor=(0.5, -0.12), 
                 ncols=5,
                 fontsize=7)

fig.savefig('./output/fig3d.svg', bbox_inches='tight')
