import pandas as pd
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import os
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)
warnings.simplefilter(action='ignore', category=UserWarning)


directory = './data/figure4_5'

data_df_biphasic = pd.read_csv(f'{directory}/figure4e.csv')
data_df_arbitrary = pd.read_csv(f'{directory}/figure5d.csv')

timing_df_biphasic = pd.read_csv(f'{directory}/figure4f.csv')
timing_df_arbitrary = pd.read_csv(f'{directory}/figure5e.csv')


os.makedirs('./output/', exist_ok=True)



def plot_quantity(data, quantity, figsize=(1.5, 3), ylim=None, log=False,
                  y_points=None, ylabels=None, strip=True, legend=False):
    fig = plt.figure(dpi=200, figsize=figsize)
    g = sns.boxplot(#showmeans=True,
                    #meanline=True,
                    meanprops={'color': 'k', 'ls': '-', 'lw': 6},
                    #medianprops={'visible': False},
                    #whiskerprops={'visible': False},
                    zorder=5,
                    x="method",
                    y=quantity,
                    data=data,
                    showfliers=False,
                    boxprops={'facecolor':'none'},
                    #showbox=False,
                    #showcaps=False,
                    linewidth=1.3
                   )
    if strip:
        sns.stripplot(data=data, zorder=20, x="method", y=quantity, hue="species",
                      hue_order=['pig', 'human'], palette='Set1', size=3, ax=g)
    else:
        sns.lineplot(data=data, zorder=20, x="method", y=quantity, units='id',
                     hue="species", hue_order=['pig', 'human'], palette='Set1',
                     size=4, ax=g, estimator=None, marker='o', markersize=3)
    sns.despine(offset=5)
    plt.xticks(rotation=45, ha='right')
    if ylim is not None:
        plt.ylim(*ylim)
    if not legend:
        plt.gca().get_legend().remove()
    else:
        plt.legend(bbox_to_anchor=(1.04, 1))
    plt.xlabel('')
    if y_points is not None:
        for y, yl in zip(y_points, ylabels):
            plt.axhline(y, linestyle='--', linewidth=0.5, zorder=1, color='k', alpha=0.6)
            plt.text(plt.gca().get_xlim()[1], y, yl, ha='right', va='bottom', fontsize=7, color='black')
    if log:
        plt.yscale('log')
    return fig


# performance biphasic

fig = plot_quantity(data_df_biphasic, "% on target", strip=True, ylim=[-5, 105], legend=False)
fig.savefig('./output/fig4e_1.svg', bbox_inches='tight', transparent=True)

fig = plot_quantity(data_df_biphasic, "% off target", strip=True, ylim=[-5, 105], legend=False)
fig.savefig('./output/fig4e_2.svg', bbox_inches='tight', transparent=True)

fig = plot_quantity(data_df_biphasic, "WBCE", strip=True, legend=False)
fig.savefig('./output/fig4e_3.svg', bbox_inches='tight', transparent=True)

# timing biphasic

fig = plot_quantity(timing_df_biphasic, "time (s)", log=True,
                    y_points=[1, 60, 300, 60*60, 60*60*24, 60*60*24*7],
                    ylabels=['1 s', '1 min', '5 min', '1 hr', '1 day', '1 week'], legend=True)
fig.savefig('./output/fig4f_1.svg', bbox_inches='tight', transparent=True)

total_times = {
    'method': [],
    'total time (s)': []
}

for m in timing_df_biphasic['method'].unique():
    t = timing_df_biphasic[timing_df_biphasic['method']==m]
    total_times['method'].append(m)
    if m == 'GD + S-MF (1 CPU, 1 GPU)':
        total_times['total time (s)'].append(165)
    else:
        total_times['total time (s)'].append(t['time (s)'].sum())

total_times_df = pd.DataFrame(total_times)

fig = plt.figure(dpi=200, figsize=(1.5, 3))
sns.barplot(data=total_times_df, x="method", y="total time (s)", facecolor=sns.color_palette()[2])
plt.yscale('log')
y_points=[60, 60*5, 60*60, 60*60*5, 60*60*24, 60*60*24*7]
ylabels = ['1 min', '5 min', '1 hr', '5 hr', '1 day', '1 week']
for y, yl in zip(y_points, ylabels):
    plt.axhline(y, linestyle='--', linewidth=0.5, zorder=1, color='k', alpha=0.6)
    plt.text(plt.gca().get_xlim()[1], y, yl, ha='right', va='bottom', color='black', fontsize=7)
sns.despine(offset=5)
plt.xticks(rotation=45, ha='right')
plt.xlabel('')

fig.savefig('./output/fig4f_2.svg', bbox_inches='tight', transparent=True)


# performance arbitrary

fig = plot_quantity(data_df_arbitrary, "% on target", figsize=(0.75, 3), strip=True, ylim=[-5, 105], legend=False)
fig.savefig('./output/fig5d_1.svg', bbox_inches='tight', transparent=True)

fig = plot_quantity(data_df_arbitrary, "% off target", figsize=(0.75, 3), strip=True, ylim=[-5, 105], legend=False)
fig.savefig('./output/fig5d_2.svg', bbox_inches='tight', transparent=True)

fig = plot_quantity(data_df_arbitrary, "WBCE", strip=True, figsize=(0.75, 3), legend=True)
fig.savefig('./output/fig5d_3.svg', bbox_inches='tight', transparent=True)

fig = plot_quantity(timing_df_arbitrary, "time (s)", log=True,
                    y_points=[60, 300, 60*60, 60*60*24, 60*60*24*7],
                    ylabels=['1 min', '5 min', '1 hr', '1 day', '1 week'],
                    legend=True)
fig.savefig('./output/fig5e_1.svg', bbox_inches='tight', transparent=True)

total_times_arbitrary = {
    'method': [],
    'total time (s)': []
}

for m in timing_df_arbitrary['method'].unique():
    t = timing_df_arbitrary[timing_df_arbitrary['method']==m]
    total_times_arbitrary['method'].append(m)
    if m == 'GD + S-MF (1 CPU, 1 GPU)':
        total_times_arbitrary['total time (s)'].append(165)
    else:
        total_times_arbitrary['total time (s)'].append(t['time (s)'].sum())

fig = plt.figure(dpi=200, figsize=(1.5, 3))
sns.barplot(data=total_times_arbitrary, x="method", y="total time (s)", facecolor=sns.color_palette()[2])
plt.yscale('log')
y_points=[60, 60*5, 60*60*24, 60*60*24*30]
ylabels = ['1 min', '5 min', '1 day', '1 month']
for y, yl in zip(y_points, ylabels):
    plt.axhline(y, linestyle='--', linewidth=0.5, zorder=1, color='k', alpha=0.6)
    plt.text(plt.gca().get_xlim()[1], y, yl, ha='right', va='bottom', color='black', fontsize=7)
sns.despine(offset=5)
plt.xticks(rotation=45, ha='right')
plt.xlabel('')

fig.savefig('./output/fig5e_2.svg', bbox_inches='tight', transparent=True)


# stats

import scipy

def test(x, y, df1=data_df_biphasic, df2=data_df_biphasic):
    return scipy.stats.wilcoxon(
        df1[df1['method']==x]['WBCE'].to_numpy(),
        df2[df2['method']==y]['WBCE'].to_numpy()
    )

comps = [
    ('DE + NEURON', 'DE + S-MF', data_df_biphasic, data_df_biphasic, 'bi, bi'),
    ('DE + NEURON', 'GD + S-MF', data_df_biphasic, data_df_biphasic, 'bi, bi'),
    ('DE + S-MF', 'GD + S-MF', data_df_biphasic, data_df_biphasic, 'bi, bi'),
    ('DE + NEURON', 'DE + NEURON', data_df_biphasic, data_df_arbitrary, 'bi, arb'),
    ('DE + NEURON', 'GD + S-MF', data_df_biphasic, data_df_arbitrary, 'bi, arb'),
    ('DE + S-MF', 'DE + NEURON', data_df_biphasic, data_df_arbitrary, 'bi, arb'),
    ('DE + S-MF', 'GD + S-MF', data_df_biphasic, data_df_arbitrary, 'bi, arb'),
    ('GD + S-MF', 'GD + S-MF', data_df_biphasic, data_df_arbitrary, 'bi, arb'),
    ('GD + S-MF', 'DE + NEURON', data_df_biphasic, data_df_arbitrary, 'bi, arb'),
    ('DE + NEURON', 'GD + S-MF', data_df_arbitrary, data_df_arbitrary, 'arb, arb'),
]

pvals = []
indices = []
W = []
cond = []

for i, (x, y, df1, df2, c) in enumerate(comps):
    xdat = df1[df1['method']==x]['WBCE'].to_numpy()
    ydat = df2[df2['method']==y]['WBCE'].to_numpy()
    W.append(test(x, y, df1, df2).statistic)
    pvals.append(test(x, y, df1, df2).pvalue)
    cond.append(c)
    indices.append(i)

pvals = np.array(pvals)
W = np.array(W)
cond = np.array(cond)

id_sort = np.argsort(pvals)
ranked_p = pvals[id_sort]
ranked_W = W[id_sort]
ranked_cond = cond[id_sort]

corrected_p = []
significant = []
for i, p in enumerate(ranked_p):
    corrected = 0.05 / (len(ranked_p) - i)
    corrected_p.append(corrected)
    significant.append(p < corrected)

for r, i, c, w, cc in zip(ranked_p, id_sort, corrected_p, ranked_W, ranked_cond):
    f, l = cc.split(', ')
    print(i, comps[i][0] + ' ' + f, comps[i][1] + ' ' + l, r, c, w)
