import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import os

os.makedirs('./output/', exist_ok=True)
data = pd.read_csv('./data/supplement/supplement_figure10.csv')


# supplementary figure 10a

import numpy as np
from sklearn.metrics import r2_score

r2 = r2_score(data['sundtgpu threshold'], data['threshold (mA)'])

fig = plt.figure(dpi=200, figsize=(3, 3))
imthera_data = data[(data['cuff']=='ImThera')]

imth_thr = imthera_data['threshold (mA)'].to_numpy()
imth_sundt_thr = imthera_data['sundtgpu threshold'].to_numpy()

plt.scatter(imth_sundt_thr, imth_thr, s=0.2, zorder=20, alpha=0.8, label='ImThera')
plt.plot(np.arange(0, 501, 1), np.arange(0, 501, 1), 'lightgrey', linewidth=1)


plt.axis('square')
plt.text(50, 300, f'$R^2$ = {r2:.3f}')
plt.ylabel('NEURON threshold (mA)')
plt.xlabel('surrogate threshold (mA)')
sns.despine(offset=6, trim=False)
# plt.legend()

fig.savefig('./output/sfig10_a.svg', bbox_inches='tight')


# supplementary figure 10b

import numpy as np

fig = plt.figure(dpi=200, figsize=(3, 3))

colors = sns.color_palette('colorblind')[:2]
styles = ['-', '--']
cuff = ['ImThera', 'Helical']
species = ['Human', 'Pig']

ax = sns.lineplot(x=r"diameter $(\mu m)$", y=r"absolute percentage error (%)",
                 hue='species', data=data,
                 palette="colorblind", hue_order = ['human', 'pig'],
                 errorbar='ci', legend=False)

ax2 = ax.twinx()
for ss, sty in enumerate(colors):
    ax2.plot(np.NaN, np.NaN,
             label=species[ss], c=sty)
ax2.get_yaxis().set_visible(False)
ax2.legend(loc='upper left', bbox_to_anchor=(1, 1))

ax.set_ylim(0, 0.5)
ax.set_xlabel("fiber diameter (μm)")
ax.set_xticks([1, 2, 3])
sns.despine(offset=6, trim=False)

fig.savefig('./output/sfig10_b.svg', bbox_inches='tight')


# supplementary figure 10c

fig, ax = plt.subplots(1, 1, dpi=200, figsize=(3, 3))

titles = ['ImThera', 'Helical']

ax.axhline(-5, linestyle='--', linewidth=1, color='black', alpha=0.6)
ax.axhline(5, linestyle='--', linewidth=1, color='black', alpha=0.6)
ax.axhline(-10, linestyle='--', linewidth=1, color='red', alpha=0.6)
ax.axhline(10, linestyle='--', linewidth=1, color='red', alpha=0.6)
ax.axhline(0, linestyle='--', linewidth=1, color='green', alpha=0.6)
#ax.tick_params(axis='x', labelsize=12)
#ax.tick_params(axis='y', labelsize=12)
#ax.set_title(t)

g = sns.violinplot(x=r"diameter $(\mu m)$", y=r"threshold error (\%)",
                   hue='species', data=data[(data['cuff']=='ImThera')],
                   palette="colorblind", showfliers=False, split=True, 
                   hue_order = ['human', 'pig'], cut=0, ax=ax,
                   legend=False, gap=0.8)
sns.despine(offset=6, trim=False)
g.set_ylabel("threshold error (%)")
    
ax.xaxis.set_ticklabels([1, 2, 3])
g.set_xlabel("fiber diameter (μm)")

ax.set_ylim(-10.5, 5.5)
ax.get_legend().remove()

fig.add_subplot(111, frameon=False)
# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', which='both', top=False, bottom=False, left=False, right=False)
    
from matplotlib.lines import Line2D
from matplotlib.patches import Patch

cmap = sns.color_palette('colorblind')

custom_lines = [Patch(facecolor=cmap[0], edgecolor='black'),
                Patch(facecolor=cmap[1], edgecolor='black')]
plt.gca().legend(custom_lines, ['Human', 'Pig'], loc='upper center', bbox_to_anchor=(0.5, -0.22), ncols=2)

fig.savefig('./output/sfig10_c.svg', bbox_inches='tight')


# supplementary figure 10d

fig, ax = plt.subplots(1, 1, dpi=200, figsize=(3, 3))

titles = ['ImThera', 'Helical']

ax.axhline(-5, linestyle='--', linewidth=1, color='black', alpha=0.6)
ax.axhline(5, linestyle='--', linewidth=1, color='black', alpha=0.6)
ax.axhline(-10, linestyle='--', linewidth=1, color='red', alpha=0.6)
ax.axhline(10, linestyle='--', linewidth=1, color='red', alpha=0.6)
ax.axhline(0, linestyle='--', linewidth=1, color='green', alpha=0.6)
#ax.tick_params(axis='x', labelsize=12)
#ax.tick_params(axis='y', labelsize=12)
#ax.set_title(t)

g = sns.violinplot(x="stim", y=r"threshold error (\%)",
                   data=data[(data['cuff']=='ImThera')],
                   color=sns.color_palette('colorblind')[2], showfliers=True,
                   whis=5, cut=0, ax=ax, bw=0.3,
                   legend=False)
sns.despine(offset=6, trim=False)
g.set_ylabel("")
    
g.set_xlabel("")

g.set_yticklabels("")

ax.set_ylim(-10.5, 5.5)
#ax.get_legend().remove()
    
ax.set_xticks(ax.get_xticks(), ax.get_xticklabels(), rotation=45, ha='right')


fig.add_subplot(111, frameon=False)
# hide tick and tick label of the big axis
plt.tick_params(labelcolor='none', which='both', top=False, bottom=False, left=False, right=False)

fig.savefig('./output/sfig10_d.svg', bbox_inches='tight')
