import pickle
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import os

data_df = pd.read_csv('./data/supplement/supplement_figure7a-c.csv')
acc_df = pd.read_csv('./data/supplement/supplement_figure7d-f.csv')


os.makedirs('./output/', exist_ok=True)


i = 0
letter = ['a', 'b', 'c']
for q in ["% target active", "% off-target active", "WBCE"]:
    if i == 2:
        l = True
    else:
        l = False
    g = sns.catplot(
        data=data_df, x="method", y=q, col="optimization",
        kind="point", hue='nerve', height=4, aspect=.45,
        palette=sns.color_palette('Reds', 6)+sns.color_palette('Blues', 6),
        legend=l
    )
    g.set_axis_labels("", q, size=16)
    g.despine(offset=6)
    g.set_xticklabels(["S-MF", "NEURON"], rotation=45, ha='right', fontsize=16)
    g.set_titles("{col_name}", size=16)
    g.savefig(f'./output/sfig7_{letter[i]}.svg', bbox_inches='tight')
    i += 1


g = sns.catplot(
        data=acc_df, x="optimization", y='accuracy',
        kind="box", height=4, aspect=.7,
        legend=l, showfliers=False, meanprops={'color': 'k', 'ls': '-', 'lw': 1},boxprops={'facecolor':'none'},
        linewidth=3
    )
sns.stripplot(data=acc_df, zorder=20, x="optimization", y='accuracy', hue="species",
            hue_order=['pig', 'human'], palette='Set1', size=5)
plt.legend(bbox_to_anchor=(1.04, 1), fontsize=16)
g.set_axis_labels("", 'accuracy (%)', size=16)
g.despine(offset=6)
g.set_xticklabels(["DE Biphasic", "GD Biphasic", "GD Arbitrary"], rotation=45, ha='right', fontsize=16)
g.savefig('./output/sfig7_f.svg', bbox_inches='tight')


g = sns.catplot(
        data=acc_df, x="optimization", y='# incorrect',
        kind="box", height=4, aspect=.7,
        legend=l, showfliers=False, meanprops={'color': 'k', 'ls': '-', 'lw': 2},boxprops={'facecolor':'none'},linewidth=3
    )
sns.stripplot(data=acc_df, zorder=20, x="optimization", y='# incorrect', hue="species",
            hue_order=['pig', 'human'], palette='Set1', size=5, legend=False)
g.set_axis_labels("", '# incorrect', size=16)
g.despine(offset=6)
g.set_xticklabels(["DE Biphasic", "GD Biphasic", "GD Arbitrary"], rotation=45, ha='right', fontsize=16)
g.savefig('./output/sfig7_d.svg', bbox_inches='tight')


g = sns.catplot(
        data=acc_df, x="optimization", y='jaccard',
        kind="box", height=4, aspect=.7,
        legend=l, showfliers=False, meanprops={'color': 'k', 'ls': '-', 'lw': 2},boxprops={'facecolor':'none'},linewidth=3
    )
sns.stripplot(data=acc_df, zorder=20, x="optimization", y='jaccard', hue="species",
            hue_order=['pig', 'human'], palette='Set1', size=5, legend=False)
g.set_axis_labels("", 'Jaccard similarity', size=16)
g.despine(offset=6)
g.set_xticklabels(["DE Biphasic", "GD Biphasic", "GD Arbitrary"], rotation=45, ha='right', fontsize=16)
g.savefig('./output/sfig7_e.svg', bbox_inches='tight')
