import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import os

os.makedirs('./output/', exist_ok=True)

# supplementary figure 9a

df = pd.read_csv('./data/supplement/supplement_figure9a.csv')

fig = plt.figure(dpi=150, figsize=(4, 3))
sns.violinplot(x=r"diameter $(\mu m)$", y=r"threshold error (\%)",
            hue='species', data=df,
            palette="colorblind", showfliers=True,
            whis=5, hue_order = ['human', 'pig'], split=True, cut=0)
sns.despine(offset=6, trim=False)
plt.axhline(-5, linestyle='--', linewidth=0.4, color='black', alpha=0.6)
plt.axhline(5, linestyle='--', linewidth=0.4, color='black', alpha=0.6)
plt.axhline(-10, linestyle='--', linewidth=0.4, color='red', alpha=0.6)
plt.axhline(10, linestyle='--', linewidth=0.4, color='red', alpha=0.6)
plt.axhline(0, linestyle='--', linewidth=0.4, color='green', alpha=0.6)
plt.ylabel('threshold error (%)')
plt.legend(bbox_to_anchor=(1.02, 0.8), loc='upper left', borderaxespad=0, title='Species')
fig.savefig('./output/sfig9_a.svg', bbox_inches='tight')


# supplementary figure 9a

df = pd.read_csv('./data/supplement/supplement_figure9b.csv')

fig = plt.figure(dpi=200, figsize=(6, 3))
sns.violinplot(x=r"diameter $(\mu m)$", y=r"threshold error (\%)",
            data=df,
            color=sns.color_palette('colorblind')[2], showfliers=True)
sns.despine(offset=6, trim=False)
plt.axhline(-5, linestyle='--', linewidth=0.4, color='black', alpha=0.6)
plt.axhline(5, linestyle='--', linewidth=0.4, color='black', alpha=0.6)
plt.axhline(-10, linestyle='--', linewidth=0.4, color='red', alpha=0.6)
plt.axhline(10, linestyle='--', linewidth=0.4, color='red', alpha=0.6)
plt.axhline(0, linestyle='--', linewidth=0.4, color='green', alpha=0.6)
plt.ylim(-15, 15)
plt.ylabel('threshold error (%)')
plt.gca().xaxis.set_ticklabels([5.7, 6, 7, 8, 9, 10, 12, 14])
fig.savefig('./output/sfig9_b.svg', bbox_inches='tight')
